# Copyright (C) 2009 Canonical Ltd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import os, sys
from PyQt4 import QtCore, QtGui

from bzrlib.plugins.explorer.lib import (
    bookmarks,
    kinds,
    location,
    )
from bzrlib.plugins.explorer.lib.i18n import gettext, N_
from bzrlib.plugins.explorer.widgets import (
    conditional_dataview,
    )


class WelcomeModel(location._AbstractLocationModel):
    """A welcome page."""

    def __init__(self, location, action_callback, bookmark_stores, actions):
        self.location = location
        self._action_callback = action_callback
        self._bm_stores = bookmark_stores
        self._actions = actions

    def kind(self):
        """What's the kind of this location?"""
        return kinds.WELCOME_PAGE

    def display_name(self):
        """Return a name suitable for displaying in a tab or title bar."""
        return self.location

    def display_icon(self):
        return kinds.icon_for_kind(self.kind())

    def view(self):
        """Return the widget to display for this location.

        :result: a QWidget
        """
        header_layout = QtGui.QHBoxLayout()
        welcome_text = gettext("Welcome to Bazaar Explorer. To get started, "
                               "select an action below.")
        welcome_label = QtGui.QLabel(welcome_text)
        header_layout.addWidget(welcome_label)

        # Build the actions panel
        actions_bar = QtGui.QToolBar()
        actions_bar.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
        for a in self._actions:
            if a is None:
                actions_bar.addSeparator()
            else:
                actions_bar.addAction(a)
        actions_layout = QtGui.QVBoxLayout()
        actions_layout.addWidget(actions_bar)
        actions_view = QtGui.QGroupBox(gettext("Getting Started"))
        actions_view.setLayout(actions_layout)

        # Build the bookmarks panel
        bm_dataview = conditional_dataview.QBzrConditionalDataView("list",
                True, "", None)
        bm_dataview.connect(bm_dataview.view(),
            QtCore.SIGNAL("doubleClicked(QModelIndex)"),
            self.do_bookmark_selected)
        bm_layout = QtGui.QVBoxLayout()
        bm_layout.addWidget(bm_dataview)
        bm_view = QtGui.QGroupBox(gettext("Bookmarks"))
        bm_view.setLayout(bm_layout)

        # Put them all together
        layout = QtGui.QVBoxLayout()
        layout.addLayout(header_layout)
        layout.addWidget(actions_view)
        layout.addWidget(bm_view)
        widget = QtGui.QWidget()
        widget.setLayout(layout)

        # Populate the bookmarks panel
        self.bm_dataview = bm_dataview
        self.refresh_view()
        return widget

    def do_bookmark_selected(self, model_index):
        model = model_index.model()
        row = model_index.row()
        bm = self.bookmarks_by_row[row]
        self._action_callback("open-location", bm.location)

    def refresh_view(self):
        # Build the bookmarks data model
        self.bookmarks_by_row = []
        bm_tuple_list = []
        bm_icon_list = []
        for bm_store in self._bm_stores:
            self._append_bookmarks(bm_store.root(), bm_tuple_list, bm_icon_list)
        bm_decorator_provider = lambda row, record: bm_icon_list[row]
        self.bm_dataview.setData(bm_tuple_list, bm_decorator_provider)

    def _append_bookmarks(self, bm_folder, bm_tuple_list, bm_icon_list):
        for entry in bm_folder:
            if isinstance(entry, bookmarks.Bookmark):
                self.bookmarks_by_row.append(entry)
                bm_tuple_list.append((entry.title,))
                bm_icon_list.append(kinds.icon_for_kind(entry.kind))
            elif isinstance(entry, bookmarks.BookmarkFolder):
                # TODO: Nest folders, don't flatten them
                self._append_bookmarks(entry, bm_tuple_list, bm_icon_list)
