# Copyright (C) 2009 Canonical Ltd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sys

from PyQt4 import QtCore, QtGui

from bzrlib.plugins.explorer.lib.i18n import gettext, N_
from bzrlib.plugins.explorer.lib.kinds import (
    OPEN_FOLDER_ACTION,
    EDIT_ACTION,
    icon_for_kind,
    )


class WorkingTreeBrowser(QtGui.QDockWidget):

    def __init__(self, action_callback, browse_action, *args):
        self._action_callback = action_callback
        self._browse_action = browse_action
        QtGui.QDockWidget.__init__(self, *args)
        self.setObjectName("WorkingTreeDockWidget")
        self.setWidget(self._build_ui())
        self._root = None
        self._selected_fileinfo = None

    def _build_ui(self):
        # Init the model and view
        model = QtGui.QDirModel()
        # sort flags for tree view: direcotries first,
        # on Windows items are case-insensitive
        flags = QtCore.QDir.SortFlags(QtCore.QDir.Name|QtCore.QDir.DirsFirst)
        if sys.platform == 'win32':
            flags |= QtCore.QDir.IgnoreCase
        model.setSorting(flags)
        view = QtGui.QTreeView()
        view.setModel(model)

        # Hide all except the Name column
        for i in  range(1, model.columnCount()):
            view.setColumnHidden(i, True)
        if getattr(view, 'setHeaderHidden', None):
            view.setHeaderHidden(True)

        # Build a button bar for the view
        browse_button = QtGui.QPushButton()
        browse_button.setText(self._browse_action.iconText())
        browse_button.setIcon(self._browse_action.icon())
        open_button = QtGui.QPushButton()
        open_button.setText(gettext("&Manage"))
        open_button.setIcon(icon_for_kind(OPEN_FOLDER_ACTION))
        edit_button = QtGui.QPushButton()
        edit_button.setText(gettext("&Edit"))
        edit_button.setIcon(icon_for_kind(EDIT_ACTION))
        buttons = QtGui.QDialogButtonBox()
        buttons.addButton(browse_button, QtGui.QDialogButtonBox.ActionRole)
        buttons.addButton(open_button, QtGui.QDialogButtonBox.ActionRole)
        buttons.addButton(edit_button, QtGui.QDialogButtonBox.AcceptRole)
        self.connect(browse_button, QtCore.SIGNAL("clicked(bool)"),
            self.do_browse_items)
        self.connect(open_button, QtCore.SIGNAL("clicked(bool)"),
            self.do_open_folder)
        self.connect(edit_button, QtCore.SIGNAL("clicked(bool)"),
            self.do_edit_file)
        self._edit_button = edit_button

        # Add the button box the view
        view_layout = QtGui.QVBoxLayout()
        view_layout.addWidget(view)
        view_layout.addWidget(buttons)
        view_panel = QtGui.QWidget()
        view_panel.setLayout(view_layout)

        # Show either the view or a message
        msg = QtGui.QTextBrowser()
        msg.setText(gettext("No working tree"))
        stacked = QtGui.QStackedWidget()
        stacked.addWidget(msg)
        stacked.addWidget(view_panel)

        # Track the selection
        self.connect(view, QtCore.SIGNAL("doubleClicked(QModelIndex)"),
            self.do_double_clicked)
        sel_model = view.selectionModel()
        self.connect(sel_model,
            QtCore.SIGNAL("currentChanged(QModelIndex,QModelIndex)"),
            self.do_current_changed)

        # Save things and return
        self.model = model
        self.view = view
        self.stacked = stacked
        return stacked

    def do_browse_items(self):
        self._action_callback("list", None)

    def do_open_folder(self):
        fileinfo = self._selected_fileinfo
        if fileinfo is None:
            folder = self._root
        elif fileinfo.isDir():
            folder = fileinfo.canonicalFilePath()
        else:
            folder = fileinfo.canonicalPath()
        self._action_callback("open", folder)

    def do_edit_file(self):
        fileinfo = self._selected_fileinfo
        path = fileinfo.canonicalFilePath()
        self._action_callback("edit", unicode(path))

    def do_double_clicked(self, model_index):
        model = model_index.model()
        fileinfo = model.fileInfo(model_index)
        if not fileinfo.isDir():
            path = fileinfo.canonicalFilePath()
            self._action_callback("open", unicode(path))

    def do_current_changed(self, model_index, prev_model_index):
        model = model_index.model()
        if model is None:
            fileinfo = None
        else:
            fileinfo = model.fileInfo(model_index)
        edit_is_enabled = True
        if fileinfo is None or fileinfo.isDir():
            edit_is_enabled = False
        self._edit_button.setEnabled(edit_is_enabled)
        self._selected_fileinfo = fileinfo

    def set_root(self, path):
        """Change the browser to display the tree from path down.

        :param path: if None, the view is cleared.
        """
        if path is None:
            # Show the message
            self.stacked.setCurrentIndex(0)
        else:
            # Show the tree
            self.stacked.setCurrentIndex(1)
            index = self.model.index(path)
            self.view.setRootIndex(index)
            self.view.selectionModel().reset()
        self._root = path
        self._selected_fileinfo = None
        self._edit_button.setEnabled(False)

    def refresh(self):
        """Refresh the QDirModel."""
        self.model.refresh()
